#version 330

struct Light {
	vec3 lightColor;
	float ambientIntensity;
	float diffuseIntensity;
};

struct DirectionalLight {
	Light light;
	vec3 lightDirection;
};

struct PointLight {
	Light light;
	vec3 lightPosition;
	float attConst;
	float attLin;
	float attExp;
};

struct SpotLight {
	PointLight light;
	vec3 lightDirection;
	float cut;
};

in vec3 worldNormal;
in vec3 worldPos;
in vec2 fragUV;

uniform DirectionalLight dl;

uniform int pointLightCount;
uniform PointLight pointLights[2];

uniform int spotLightCount;
uniform SpotLight spotLights[1];

uniform vec3 viewWorldPos;
uniform float specularIntensity;
uniform float specularPower;

uniform sampler2D colorTexture;

out vec4 fragColor;


vec4 createLight(Light l, vec3 lightDirection, vec3 normal){
	vec4 ambientColor = vec4(l.lightColor, 1.0f) * l.ambientIntensity;
	float diffuseFactor = dot(normal, -lightDirection);

	vec4 diffuseColor = vec4(0.0f, 0.0f, 0.0f, 0.0f);
	vec4 specularColor = vec4(0.0f, 0.0f, 0.0f, 0.0f);

	if(diffuseFactor > 0.0f){
		diffuseColor = vec4(l.lightColor, 1.0f) * l.diffuseIntensity * diffuseFactor;

		vec3 toViewer = normalize(viewWorldPos - worldPos);
		vec3 reflected = normalize(reflect(lightDirection, normal));
		float specularFactor = pow(dot(toViewer, reflected), specularPower);
		if(specularFactor > 0){
			specularColor = vec4(l.lightColor, 1.0f) * specularIntensity * specularFactor;
		}

	}

	return (ambientColor + (diffuseColor + specularColor));
}

vec4 createDirectionalLight(vec3 normal){
	return createLight(dl.light, dl.lightDirection, normal);
}

vec4 createPointLight(PointLight l, vec3 normal){
	vec3 lightDirection = worldPos - l.lightPosition;
    float dist = length(lightDirection);
    lightDirection = normalize(lightDirection);
    vec4 color = createLight(l.light, lightDirection, normal);
    float att =  l.attConst + l.attLin * dist + l.attExp * dist * dist;
	
    return (color / att); 
}

vec4 createSpotLight(SpotLight l, vec3 normal){
	vec3 lightToPixel = normalize(worldPos - l.light.lightPosition);
	float spotFactor = dot(lightToPixel, l.lightDirection);

	if(spotFactor > l.cut){
		vec4 color = createPointLight(l.light, normal);
		return (color * (1.0f - (1.0f - spotFactor) * 1.0f / (1.0f - l.cut)));
	} else {
		return vec4(0.0f, 0.0f, 0.0f, 0.0f);
	}
}


void main(){
	vec3 nNormal = normalize(worldNormal);

	vec4 lightColor = createDirectionalLight(nNormal);
	for(int i = 0; i < pointLightCount; i++){
		lightColor += createPointLight(pointLights[i], nNormal);
	}
	for(int i = 0; i < spotLightCount; i++){
		lightColor += createSpotLight(spotLights[i], nNormal);
	}

	vec3 textureColor = texture(colorTexture, fragUV).rgb;

	fragColor = vec4(textureColor, 1.0f) * lightColor;
}
